@@ -31,11 +31,13 @@
 #include <libgnomevfs/gnome-vfs-application-registry.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
+#include "capplet-util.h"
 #include "mime-edit-dialog.h"
 #include "mime-types-model.h"
 
 #include "libuuid/uuid.h"
 
+#undef WID(x)
 #define WID(x) (glade_xml_get_widget (dialog->p->dialog_xml, x))
 
 enum {
@@ -98,8 +100,6 @@ static void add_ext_cb                   (MimeEditDialog *dialog);
 static void remove_ext_cb                (MimeEditDialog *dialog);
 static void choose_cat_cb                (MimeEditDialog *dialog);
 static void default_action_changed_cb    (MimeEditDialog *dialog);
-static void use_category_defaults_toggled_cb (MimeEditDialog  *dialog,
-					      GtkToggleButton *tb);
 static void response_cb                  (MimeEditDialog *dialog,
 					  gint            response_id);
 
@@ -159,12 +159,29 @@ mime_edit_editable_enters (MimeEditDialog *dialog, GtkEditable *editable)
 		GTK_WINDOW (dialog->p->dialog_win));
 }
 
+static void
+check_for_content (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
+		   gpointer data) 
+{
+	*((gboolean *)data) = TRUE;
+}
+
+static void
+remove_ext_sensitivity_cb (GtkTreeSelection *selection, GtkWidget *button)
+{
+	gboolean flag = FALSE;
+	gtk_tree_selection_selected_foreach (selection, check_for_content, &flag);
+	gtk_widget_set_sensitive (GTK_WIDGET (button), flag);
+}
+
 static void
 mime_edit_dialog_init (MimeEditDialog *dialog, MimeEditDialogClass *class)
 {
 	GtkSizeGroup *size_group;
 	GtkTreeView *view;
 	GtkCellRenderer *renderer;
+	GtkWidget *remove_ext;
+	GtkTreeSelection *selection;
 
 	dialog->p = g_new0 (MimeEditDialogPrivate, 1);
 	dialog->p->dialog_xml = glade_xml_new
@@ -193,19 +210,28 @@ mime_edit_dialog_init (MimeEditDialog *dialog, MimeEditDialogClass *class)
 		 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 		 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 		 NULL);
+	capplet_set_icon (dialog->p->dialog_win, "gnome-ccmime.png");
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog->p->dialog_win),
 		 GTK_RESPONSE_OK);
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog->p->dialog_win)->vbox), WID ("edit_widget"), TRUE, TRUE, 0);
 
-	g_signal_connect_swapped (G_OBJECT (WID ("add_ext_button")), "clicked", (GCallback) add_ext_cb, dialog);
-	g_signal_connect_swapped (G_OBJECT (WID ("remove_ext_button")), "clicked", (GCallback) remove_ext_cb, dialog);
-	g_signal_connect_swapped (G_OBJECT (WID ("choose_button")), "clicked", (GCallback) choose_cat_cb, dialog);
-	g_signal_connect_swapped (G_OBJECT (WID ("default_action_select")), "changed", (GCallback) default_action_changed_cb, dialog);
-	g_signal_connect_swapped (G_OBJECT (WID ("use_category_defaults_toggle")), "toggled",
-				  (GCallback) use_category_defaults_toggled_cb, dialog);
+	remove_ext = WID ("remove_ext_button");
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (WID ("ext_list")));
+	g_signal_connect (G_OBJECT (selection), "changed",
+		(GCallback) remove_ext_sensitivity_cb, remove_ext);
+	g_signal_connect_swapped (G_OBJECT (remove_ext), "clicked",
+		(GCallback) remove_ext_cb, dialog);
+	g_signal_connect_swapped (G_OBJECT (WID ("new_ext_entry")), "activate",
+		(GCallback) add_ext_cb, dialog);
+	g_signal_connect_swapped (G_OBJECT (WID ("choose_button")), "clicked",
+		(GCallback) choose_cat_cb, dialog);
+	g_signal_connect_swapped (G_OBJECT (WID ("default_action_select")), "changed",
+		(GCallback) default_action_changed_cb, dialog);
+	g_signal_connect_swapped (G_OBJECT (dialog->p->dialog_win), "response",
+		(GCallback) response_cb, dialog);
 
-	g_signal_connect_swapped (G_OBJECT (dialog->p->dialog_win), "response", (GCallback) response_cb, dialog);
+	remove_ext_sensitivity_cb (selection, (gpointer)remove_ext);
 
 	mime_edit_editable_enters (dialog, GTK_EDITABLE (WID ("description_entry")));
 	mime_edit_editable_enters (dialog, GTK_EDITABLE (WID ("mime_type_entry")));
@@ -292,7 +318,7 @@ mime_edit_dialog_set_prop (GObject *object, guint prop_id, const GValue *value,
 				mime_edit_dialog->p->model);
 			setup_add_dialog (mime_edit_dialog);
 			gtk_window_set_title (GTK_WINDOW (mime_edit_dialog->p->dialog_win),
-				(_("Add file type")));
+				(_("Add File Type")));
 			gtk_widget_show_all (mime_edit_dialog->p->dialog_win);
 		}
 
@@ -377,19 +403,42 @@ mime_edit_dialog_new (GtkTreeModel *model, MimeTypeInfo *info)
 }
 
 GObject *
-mime_add_dialog_new (GtkTreeModel *model, GtkWindow *parent) 
+mime_add_dialog_new (GtkTreeModel *model, GtkWindow *parent,
+		     char const *file_name) 
 {
-	GObject *obj = g_object_new (mime_edit_dialog_get_type (),
+	GObject *dialog = g_object_new (mime_edit_dialog_get_type (),
 		"model", model,	/* must be before is-add */
 		NULL);
-	g_object_set (obj,
+	g_object_set (dialog,
 		      "is-add", TRUE,
 		      NULL);
 	if (parent != NULL)
 		gtk_window_set_transient_for (
-			GTK_WINDOW (MIME_EDIT_DIALOG (obj)->p->dialog_win),
+			GTK_WINDOW (MIME_EDIT_DIALOG (dialog)->p->dialog_win),
 			parent);
-	return obj;
+
+	if (file_name != NULL) {
+		/* quick and dirty, no tests for backslashed dots */
+		char const *last = g_utf8_strrchr (file_name, -1,
+			g_utf8_get_char ("."));
+
+		if (last != NULL && last[1] != '\0') {
+			MimeTypeInfo *info = MIME_EDIT_DIALOG (dialog)->p->info;
+			char *lower = g_utf8_strdown (last +1, -1);
+
+			info->mime_type = g_strdup_printf ("application/x-%s", lower);
+			g_free (lower);
+
+			mime_type_info_set_file_extensions (info,
+				g_list_prepend (NULL, g_strdup (last+1)));
+			mime_type_info_set_category_name (info,
+				"Misc", _("Misc"), model);
+
+			fill_dialog (MIME_EDIT_DIALOG (dialog));
+		}
+	}
+
+	return dialog;
 }
 
 static void
@@ -419,7 +468,6 @@ fill_dialog (MimeEditDialog *dialog)
 		mime_type_info_get_category_name (dialog->p->info));
 
 	dialog->p->use_cat_dfl = dialog->p->info->use_category;
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("use_category_defaults_toggle")), dialog->p->use_cat_dfl);
 	update_sensitivity (dialog);
 
 	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (WID ("icon_entry")), mime_type_info_get_icon_path (dialog->p->info));
@@ -602,6 +650,109 @@ collect_filename_extensions (MimeEditDialog *dialog)
 	return ret;
 }
 
+/**
+ * mime_edit_dialog_get_app : 
+ * @glade :
+ * @mime_type : a fall back in case we can't generate a meaningful application name.
+ * @current :
+ *
+ * A utility routine for looking up applications.  it should handle life cycle
+ * and hopefully merge in existing copies of custom applications.
+ **/
+void
+mime_edit_dialog_get_app (GladeXML *glade, char const *mime_type,
+			  GnomeVFSMimeApplication **current)
+{
+	GtkWidget *menu = glade_xml_get_widget (glade, "default_action_select");
+	gint       idx = gtk_option_menu_get_history (GTK_OPTION_MENU (menu));
+	GtkWidget *shell = gtk_option_menu_get_menu (GTK_OPTION_MENU (menu));
+	GObject   *item = (g_list_nth (GTK_MENU_SHELL (shell)->children, idx))->data;
+
+	GnomeVFSMimeApplication *res, *app = g_object_get_data (item, "app");
+
+	if (app == NULL) {
+		GnomeFileEntry *program_entry =
+			GNOME_FILE_ENTRY (glade_xml_get_widget (glade, "program_entry"));
+		char *cmd = gnome_file_entry_get_full_path (program_entry, TRUE);
+		gboolean requires_terminal = gtk_toggle_button_get_active (
+			GTK_TOGGLE_BUTTON (glade_xml_get_widget (glade, "needs_terminal_toggle")));
+		char *base_cmd;
+
+		GList *ptr, *app_list = NULL;
+		
+		/* If the cmd is NULL its possible that the user just types a
+		 * command name in their path.  Because the file picker seems
+		 * to insist that it only return a valid path we ask it to
+		 * verify the app exists so that we can catch it here and just
+		 * use the raw non-absolute name here.
+		 */
+		if (cmd == NULL) {
+			char const *tmp = gtk_entry_get_text (
+				GTK_ENTRY (gnome_file_entry_gtk_entry (program_entry)));
+			cmd = g_strdup (tmp == NULL ? "" : tmp);
+		}
+		base_cmd = g_path_get_basename  (cmd);
+		if (base_cmd == NULL)
+			base_cmd = g_strdup ("");
+
+		app_list = gnome_vfs_application_registry_get_applications (NULL);
+		for (ptr = app_list; ptr != NULL ; ptr = ptr->next) {
+			char const *app_cmd = gnome_vfs_application_registry_peek_value (ptr->data,
+				GNOME_VFS_APPLICATION_REGISTRY_COMMAND);
+
+			/* Look for a matching application (with or without path) */
+			if (app_cmd != NULL &&
+			    (!strcmp (cmd, app_cmd) || !strcmp (base_cmd, app_cmd))) {
+				gboolean ok, app_req = gnome_vfs_application_registry_get_bool_value (ptr->data,
+					GNOME_VFS_APPLICATION_REGISTRY_REQUIRES_TERMINAL, &ok);
+				if (ok && app_req == requires_terminal)
+					break;
+			}
+		}
+
+		/* No existing application, lets create one */
+		if (ptr == NULL) {
+			res = g_new0 (GnomeVFSMimeApplication, 1);
+			res->command = cmd;
+			res->requires_terminal = requires_terminal;
+
+			res->name = base_cmd;
+			if (res->name != NULL && *res->name) {
+				/* Can we use the app name as the id ?
+				 * We know that there are no apps with the same
+				 * command, so if the id is taken we are screwed
+				 */
+				if (gnome_vfs_application_registry_get_mime_application (res->name) == NULL)
+					res->id = g_strdup (res->name);
+			} else { /* fail safe to ensure a name */
+				g_free (res->name);
+				res->name = g_strdup_printf ("Custom %s", mime_type);
+			}
+
+			/* If there is no id yet, make up a unique string */
+			if (res->id == NULL) {
+				uuid_t  app_uuid;
+				gchar   app_uuid_str[100];
+				uuid_generate (app_uuid);
+				uuid_unparse (app_uuid, app_uuid_str);
+				res->id = g_strdup (app_uuid_str);
+			}
+
+			gnome_vfs_application_registry_save_mime_application (res);
+		} else {
+			g_free (cmd);
+			g_free (base_cmd);
+			res = gnome_vfs_application_registry_get_mime_application (ptr->data);
+		}
+
+		g_list_free (app_list);
+	} else
+		res = gnome_vfs_mime_application_copy (app);
+
+	gnome_vfs_mime_application_free (*current);
+	*current = res;
+}
+
 static void
 store_data (MimeEditDialog *dialog) 
 {
@@ -618,8 +769,6 @@ store_data (MimeEditDialog *dialog)
 	uuid_t         mime_uuid;
 	gchar          mime_uuid_str[100];
 
-	GnomeVFSMimeApplication *app;
-
 	GtkTreeIter    iter;
 	GtkTreePath   *path;
 
@@ -654,9 +803,6 @@ store_data (MimeEditDialog *dialog)
 		dialog->p->info->small_icon_pixbuf = NULL;
 	}
 
-	dialog->p->info->use_category =
-		gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WID ("use_category_defaults_toggle")));
-
 	option_menu = GTK_OPTION_MENU (WID ("component_select"));
 	menu_shell = GTK_MENU_SHELL (gtk_option_menu_get_menu (option_menu));
 	idx = gtk_option_menu_get_history (option_menu);
@@ -665,31 +811,9 @@ store_data (MimeEditDialog *dialog)
 	CORBA_free (dialog->p->info->default_component);
 	dialog->p->info->default_component = g_object_get_data (menu_item, "component");
 
-	option_menu = GTK_OPTION_MENU (WID ("default_action_select"));
-	menu_shell = GTK_MENU_SHELL (gtk_option_menu_get_menu (option_menu));
-	idx = gtk_option_menu_get_history (option_menu);
-	menu_item = (g_list_nth (menu_shell->children, idx))->data;
-
-	app = g_object_get_data (menu_item, "app");
-	if (app != NULL) {
-		gnome_vfs_mime_application_free (dialog->p->info->default_action);
-		dialog->p->info->default_action = gnome_vfs_mime_application_copy (app);
-	} else {
-		if (!mime_type_info_using_custom_app (dialog->p->info)) {
-			gnome_vfs_mime_application_free (dialog->p->info->default_action);
-			dialog->p->info->default_action = g_new0 (GnomeVFSMimeApplication, 1);
-		} else if (!dialog->p->info->default_action) {
-			dialog->p->info->default_action = g_new0 (GnomeVFSMimeApplication, 1);
-		}
-
-		g_free (dialog->p->info->default_action->command);
-		dialog->p->info->default_action->command
-			= g_strdup (gtk_entry_get_text (GTK_ENTRY
-							(gnome_file_entry_gtk_entry
-							 (GNOME_FILE_ENTRY (WID ("program_entry"))))));
-		dialog->p->info->default_action->requires_terminal
-			= gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WID ("needs_terminal_toggle")));
-	}
+	mime_edit_dialog_get_app (dialog->p->dialog_xml,
+		dialog->p->info->mime_type,
+		&(dialog->p->info->default_action));
 
 	ext_list = collect_filename_extensions (dialog);
 	mime_type_info_set_file_extensions (dialog->p->info, ext_list);
@@ -716,30 +840,32 @@ store_data (MimeEditDialog *dialog)
 static gboolean
 validate_data (MimeEditDialog *dialog) 
 {
-	const gchar *tmp;
-	const gchar *mesg = NULL;
+	const gchar *mime_type;
+	GtkWidget *err_dialog = NULL;
 
-	tmp = gtk_entry_get_text (GTK_ENTRY (WID ("mime_type_entry")));
+	mime_type = gtk_entry_get_text (GTK_ENTRY (WID ("mime_type_entry")));
 
-	if (tmp != NULL && *tmp != '\0') {
-		if (strchr (tmp, ' ') || !strchr (tmp, '/')) {
-			mesg = _("Invalid MIME type. Please enter a valid MIME type, or "
-				 "leave the field blank to have one generated for you.");
-		} else if (dialog->p->is_add && (gnome_vfs_mime_type_is_known (tmp) ||
-						 get_mime_type_info (tmp) != NULL)) {
-			mesg = _("There already exists a MIME type of that name.");
+	if (mime_type != NULL && *mime_type != '\0') {
+		if (strchr (mime_type, ' ') || !strchr (mime_type, '/')) {
+			err_dialog = gtk_message_dialog_new (
+				GTK_WINDOW (dialog->p->dialog_win),
+				GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
+				GTK_BUTTONS_CANCEL,
+				_("Please enter a valid MIME type.  It should be of the form "
+				  "class/type and may not contain any spaces."));
+		} else if (dialog->p->is_add && gnome_vfs_mime_type_is_known (mime_type)) {
+			err_dialog = gtk_message_dialog_new (
+				GTK_WINDOW (dialog->p->dialog_win),
+				GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
+				GTK_BUTTONS_OK_CANCEL,
+				_("A MIME type with that name already exists, overwrite ?."));
 		}
 	}
 
-	if (mesg != NULL) {
-		GtkWidget *err_dialog = gtk_message_dialog_new (
-			GTK_WINDOW (dialog->p->dialog_win),
-			GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
-			GTK_BUTTONS_OK,
-			mesg);
-		gtk_dialog_run (GTK_DIALOG (err_dialog));
+	if (err_dialog) {
+		int res = gtk_dialog_run (GTK_DIALOG (err_dialog));
 		gtk_object_destroy (GTK_OBJECT (err_dialog));
-		return FALSE;
+		return res != GTK_RESPONSE_CANCEL;
 	}
 	return TRUE;
 }
@@ -842,13 +968,6 @@ default_action_changed_cb (MimeEditDialog *dialog)
 	update_sensitivity (dialog);
 }
 
-static void
-use_category_defaults_toggled_cb (MimeEditDialog *dialog, GtkToggleButton *tb)
-{
-	dialog->p->use_cat_dfl = gtk_toggle_button_get_active (tb);
-	update_sensitivity (dialog);
-}
-
 static void
 response_cb (MimeEditDialog *dialog, gint response_id) 
 {
